--------------------------------------------------------------------------------
--
-- Bloco de Controle do Processador Clepatra (8 bits)
-- Soluo hardwired, via mquina de estados
--
--  1/08/2004 - reviso para sntese em hardware. trocar processo nico
--  com muitos waits por estrutura de FSM convencional. (Calazans)
--
--  14/8 ==> mquina de estados simplificada (15 estados) (moraes)
--  15/8 ==> insero do sinal hold e codificao extendida para not/sta
--
--   FERNANDO GEHM MORAES         03/agosto/2005
--   NEY LAERT VILAR CALAZANS     18/junho/2006
--   CSAR AUGUSTO MISSIO MARCON  10/abril/2008
--
--------------------------------------------------------------------------------
library IEEE;
use IEEE.Std_Logic_1164.all;
use work.PacoteCleopatra.all;

entity BlocoDeControle is
	port
	(
		reset, clock, hold: in std_logic;
		halt: out std_logic;
		ir: in internal_bus;
		n, z, c, v: in std_logic;
		uins: out microinstrucao
	);
end BlocoDeControle;

architecture BlocoDeControle of BlocoDeControle is

	type CleoStates_type is
	(
		SIDLE,     -- estado dummy, apenas garante que borda de subida seguinte
		           --  desativao do reset faz o processamento comear.
		FETCH0,    -- estados onde se faz busca de instruo
		FETCH1,
		FETCH2,
		Sop1a,     --  busca do primeiro operando das demais instrues
		Sop1b,
		Sop2a,     -- estados para execuo das instrues lgicas aritmticas
		Sop2b,
		Sop3a,
		Sop3b,
		Sula,
		OP2jp1,    -- estados para execuo dos saltos
		OP2jp2,
		Ssalta,
		Sjsr
	);

	-- CODIFICAO DAS INSTRUCOES
	constant NOT1: std_logic_vector(3 downto 0) := x"0"; -- 2 codificaes-NOT
	constant NOT2: std_logic_vector(3 downto 0) := x"1";
	constant STA1: std_logic_vector(3 downto 0) := x"2"; -- 2 codificaes-STA
	constant STA2: std_logic_vector(3 downto 0) := x"3";
	constant LDA: std_logic_vector(3 downto 0)  := x"4";
	constant ADD: std_logic_vector(3 downto 0)  := x"5";
	constant ORL: std_logic_vector(3 downto 0)  := x"6";
	constant ANDL: std_logic_vector(3 downto 0) := x"7";
	constant JMP: std_logic_vector(3 downto 0)  := x"8";
	constant JC: std_logic_vector(3 downto 0)   := x"9";
	constant JN: std_logic_vector(3 downto 0)   := x"A";
	constant JZ: std_logic_vector(3 downto 0)   := x"B";
	constant JSR: std_logic_vector(3 downto 0)  := x"C";
	constant RTS: std_logic_vector(3 downto 0)  := x"D";
	constant JV: std_logic_vector(3 downto 0)   := x"E";
	constant HLT: std_logic_vector(3 downto 0)  := x"F";

	-- CODIFICAO DOS MODOS DE ENDEREAMENTO
	constant IM:  std_logic_vector(1 downto 0) := "00";
	constant DIR: std_logic_vector(1 downto 0) := "01";
	constant IND: std_logic_vector(1 downto 0) := "10";
	constant REL: std_logic_vector(1 downto 0) := "11";

	-- definicao das microinstrucoes em funcao dos registradores destino/origem
	--                                       wr     rd    alu  ce  rw  lnz lcv
-- mar <- pc
	constant mar_pc: microinstrucao    := ("000", "011", "111",'0','0','0','0');
-- mar <- mdr
	constant mar_mdr: microinstrucao   := ("000", "001", "100",'0','0','0','0');
-- mdr <- pmem(mar); pc++
	constant mdr_MmarP: microinstrucao := ("110", "011", "001",'1','1','0','0');
-- mdr <- pmem(mar)
	constant mdr_Mmar: microinstrucao  := ("001", "110", "111",'1','1','0','0');
-- mar <- mdr+PC
	constant mar_mdrpc: microinstrucao := ("000", "111", "000",'0','0','0','0');
-- ir  <- mdr
	constant ir_mdr: microinstrucao    := ("010", "001", "100",'0','0','0','0');
-- pc  <- mdr
	constant pc_mdr: microinstrucao    := ("011", "001", "100",'0','0','0','0');
-- pc  <- mdr + pc
	constant pc_mdrpc: microinstrucao  := ("011", "111", "000",'0','0','0','0');
-- rts <- pc
	constant rts_pc: microinstrucao    := ("101", "011", "111",'0','0','0','0');
-- nao faz nada
	constant nop: microinstrucao       := ("111", "110", "111",'0','0','0','0');

	alias i: std_logic_vector(3 downto 0) is ir(7 downto 4);   -- pedaos do IR
	alias me: std_logic_vector(1 downto 0) is ir(3 downto 2);

	signal salta, logic_arith: std_logic;      -- auxiliares
	signal EA, PE: CleoStates_type;           -- estados da mquina de estado

begin

	salta <= '1' when ((i = JC and c = '1') or (i = JV and v = '1') or
			(i = JN and n = '1') or (i = JZ and z = '1')) else '0';
	logic_arith <= '1' when  i = LDA or i = ADD or i = ORL or i = ANDL else '0';

	process(clock, reset)  -- processo para implementar registrador de estado
	begin
		if reset = '1' then
			EA <= Sidle;   -- Machine stays idle while processor is being reset
			halt <= '0';
		elsif clock'event and clock = '1' then
			if i = HLT then
				halt <= '1';              -- avisa que trancou
			elsif hold = '0' then
				EA <= PE;                -- O PROCESSADOR TRANCA EM HOLD!!
			end if;
		end if;
	end process;

	-- processo para implementar funes de sada e prximo estado
	process(hold, i, salta, EA, logic_arith, me)
	begin
		case EA is
			when SIDLE =>
				uins <= nop;
				PE <= FETCH0;               -- no faz nada em SIDLE

		--- BUSCA DA INSTRUO
			when FETCH0 =>
				uins <= mar_pc;         -- MAR <- PC   1o ciclo da busca
					PE <= FETCH1;

			when FETCH1 =>
				uins <= mdr_MmarP;  -- MDR <- MEM(MAR); PC++  2o ciclo da busca
				PE <= FETCH2;

			when FETCH2 =>
				uins <= ir_mdr;             -- IR <- MDR   carga do ir
				case i is
					when RTS | NOT1 | NOT2 | HLT =>
						PE <= Sula;

					when others =>
						PE <= Sop1a;
				end case;

		--- BUSCA DO OPERANDO DA INSTRUO
			when Sop1a =>
				uins <= mar_pc;                 -- MAR <- PC
				PE <= Sop1b;

			when Sop1b =>
				uins <= mdr_MmarP;                -- MDR <- MEM(MAR); PC++
				if logic_arith = '1' and ME = IM then
					PE <= Sula;
				elsif (logic_arith = '1' and (ME = DIR or ME = IND or ME = REL))
						or (i = STA1 or i = STA2) then
					PE <= Sop2a;
				elsif (i = JMP  or salta = '1') and
						(ME = IM or ME = DIR or ME = REL) then
					PE <= Ssalta;
				elsif (i = JMP  or salta = '1') and ME = IND then
					PE <= OP2jp1;
				elsif i = JSR then
					PE <= Sjsr;
				else
					PE <= FETCH0;   -- volta para o FETCH
				end if;

	-- INSTRUES LGICAS ARITMTICAS -- lda, and, or, add, not + STA + RTS
			when Sop2a =>
				case ME is
					when REL =>
						uins <= mar_mdrpc;              -- MAR <- MDR + PC
					when others =>
						uins <= mar_mdr;                -- MAR <- MDR
				end case;
				if (i = STA1 or i = STA2) and (ME = REL or ME = DIR) then
					PE <= Sula;
				else
					PE <=  Sop2b;
				end if;

			when Sop2b =>
				uins <= mdr_Mmar;          -- MDR <- MEM(MAR)
				case ME is
					when IND =>
						PE <= Sop3a;
					when others =>
						PE <=  Sula;
				end case;

			when Sop3a =>
				uins <= mar_mdr;            -- MAR <- MDR
				case i is
					when STA1 | STA2 =>
						PE <= Sula;
					when others =>
						PE <= Sop3b;
				end case;

			when Sop3b =>
				uins <= mdr_Mmar;           -- MDR <- MEM(MAR)
				PE <= Sula;

			when Sula =>
				case i is
					when LDA =>       -- ac <- mdr; lnz
						uins <= ("100", "001", "100", '0', '0', '1','0');
					when ADD =>       -- ac <- ac + mdr;lnz;lcv
						uins <= ("100", "110", "000", '0', '0', '1','1');
					when ORL =>       -- ac <- ac or mdr; lnz
						uins <= ("100", "110", "101", '0', '0', '1','0');
					when ANDL =>       -- ac <- ac and mdr; lnz
						uins <= ("100", "110", "110", '0', '0', '1','0');
					when NOT1 | NOT2 =>       -- ac <- not ac ; lnz
						uins <= ("100", "100", "010", '0', '0', '1','0');
					when RTS =>       -- pc <- rts
						uins <= ("011", "101", "111", '0', '0', '0','0');
					when STA1 | STA2 =>        -- pmem(mar) <- ac
						uins <= ("111", "100", "111", '1', '0', '0','0');
					when others =>     -- halt
						uins <= nop;
				end case;
				PE <= FETCH0;

	--- AES ESPECFICAS DAS INSTRUO DE SALTO
			when Sjsr =>
				uins <= rts_pc;                       -- RS <- PC
				case ME is
					when IND =>
						PE <= OP2jp1;

					when others =>
					PE <= Ssalta;
				end case;

			when OP2jp1 =>
				uins <= mar_mdr;                      -- MAR <- MDR
				PE <= OP2jp2;

			when OP2jp2 =>
				uins <= mdr_Mmar;                     -- MDR <- MEM(MAR)
				PE <=  Ssalta;

			when Ssalta =>
				case ME is
					when REL =>
						uins <= pc_mdrpc;             -- PC <- MDR + PC

					when others =>
						uins <= pc_mdr;               -- PC <- MDR
				end case;
				PE <= FETCH0;
		end case;
	end process;

end BlocoDeControle;