-------------------------------------------------------------------------------
--
-- Display Interface
--
--   FERNANDO GEHM MORAES         03/agosto/2005
--   NEY LAERT VILAR CALAZANS     18/junho/2006
--   CSAR AUGUSTO MISSIO MARCON  10/abril/2008
--
-------------------------------------------------------------------------------
-- This module implements the interface hardware needed to drive some
-- Digilent boards four digit seven segment display. This
-- display is multiplexed (see the specific board Reference Manual for details)
-- requiring that just one digit be displayed at any moment.
-- Examples boards are D2SB/DIO4, Spartan3 Starter Kit and Nexys
--
-- The inputs of the module are:
--		clock - the 50MHz system board clock
--		reset - the active-high system reset signal
--    di vectors - 4 vectors, each with 6 bits, where:
--		di(0) is the decimal point (active-low)
--		di(4 downto 1) is the binary value of the digit
--		di(5) is the (active-high) enable signal of the digit
--      here, i varies from 4 to 1, 4 corresponds to the rightmost
--		  digit of the display and 1 corresponds to the leftmost digit
--
-- The outputs of the module are:
--		an (4 downto 1) - the four wire active-low anode vector.
-- 			In this circuit, exactly one of these 4 wires is at logic 0
--			at any moment. The wire in 0 lights up one of the 4 7-segment
--			displays. 4 is the rightmost display while 1 is the leftmost.
--		dec_ddp (7 downto 0) - is the decoded value of the digit to show
--			at the current instant. dec_ddp(7 downto 1)  corresponds
--			respectively to the segments a b c d e f g, and dec_ddp(0) is
--			the decimal point.
--
-- Functional description: The 50MHz is divided to obtain the
--		1KHz display refresh clock. Upon reset all displays are turned
--		off. The 1KHz clock feeds a 2-bit counter. This counter
--		generates a signal to select one of the four di vectors. This
--		vector is in turn used to enable or not to show the digit in
--		question (through di(5)) and furnishes the digit value for the
--		single multiplexed 7-segment decoder. All outputs are registered
--		using the 1KHz clock.
-------------------------------------------------------------------------------
library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Std_Logic_Unsigned.all;

entity dio4_dspl_drv is
	port
	(
		clock: in std_logic;
		reset: in std_logic;
		d4: in std_logic_vector(6 downto 0);
		d3: in std_logic_vector(6 downto 0);
		d2: in std_logic_vector(6 downto 0);
		d1: in std_logic_vector(6 downto 0);
		an: out std_logic_vector(4 downto 1);
		dec_ddp: out std_logic_vector(7 downto 0)
	);
end dio4_dspl_drv;

architecture dio4_dspl_drv of dio4_dspl_drv is

	signal ck_1KHz: std_logic;
	signal dig_selection: std_logic_vector(1 downto 0);
	signal selected_dig: std_logic_vector(5 downto 0);

begin

	process(reset, clock)     -- 1KHz clock generation
		variable count_25K: integer range 0 to 25000;
	begin
		if reset = '1' then
			count_25K := 0;
			ck_1KHz <= '0';
		elsif clock'event and clock = '1' then
			count_25K := count_25K + 1;
			if count_25K = 24999 then
				count_25K := 0;
				ck_1KHz <= not ck_1KHz;
			end if;
		end if;
	end process;

	process(reset, ck_1KHz)   -- 1KHz counter to select digit and register
	begin
		if reset = '1' then
			selected_dig <= "000000";
			dig_selection <= (others => '0');
			an <= (others => '1');		  -- Disable all displays
		elsif ck_1KHz'event and ck_1KHz = '1' then
			dig_selection <= dig_selection(0) & not dig_selection (1);
			if dig_selection = "00" then
				selected_dig <= d1(5 downto 0);
				an <= "111" & (not d1(6));
			elsif dig_selection = "01" then
				selected_dig <= d2(5 downto 0);
				an <= "11" & (not d2(6)) & "1";
			elsif dig_selection = "10" then
				selected_dig <= d3(5 downto 0);
				an <= "1" & (not d3(6)) & "11";
			else
				selected_dig <= d4(5 downto 0);
				an <= (not d4(6)) & "111";
			end if;
		end if;
	end process;

	with selected_dig (5 downto 1) select   -- digit 4-to-hex decoder
		dec_ddp(7 downto 1) <=
			"0000001" when "00000", --0
			"1001111" when "00001", --1
			"0010010" when "00010", --2
			"0000110" when "00011", --3
			"1001100" when "00100", --4
			"0100100" when "00101", --5
			"0100000" when "00110", --6
			"0001111" when "00111", --7
			"0000000" when "01000", --8
			"0000100" when "01001", --9
			"0001000" when "01010", --A
			"1100000" when "01011", --b
			"0110001" when "01100", --C
			"1000010" when "01101", --d
			"0110000" when "01110", --E
			"0111000" when "01111", --F
			"0011000" when "10000", --P
			"1001000" when "10001", --H
			"1000001" when "10010", --U
			"1100011" when "10011", --u
			"1111111" when others;
	dec_ddp(0) <= selected_dig(0);  -- and the decimal point

end dio4_dspl_drv;
